@@ -21,7 +21,6 @@ import java.util.Collection;
 import java.util.List;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.atomic.AtomicReference;
-import java.util.stream.Collectors;
 
 import org.junit.After;
 import org.junit.Assert;
@@ -128,14 +127,14 @@ public class CodeMiningProjectionViewerTest {
 	protected List<Shell> getCurrentShells() {
 		return Arrays.stream(fParent.getDisplay().getShells())
 				.filter(Shell::isVisible)
-				.collect(Collectors.toList());
+				.toList();
 	}
 
 	protected List<Shell> findNewShells(Collection<Shell> beforeShells) {
 		return Arrays.stream(fParent.getDisplay().getShells())
 				.filter(Shell::isVisible)
 				.filter(shell -> !beforeShells.contains(shell))
-				.collect(Collectors.toList());
+				.toList();
 	}
 
 	protected Shell findNewShell(Collection<Shell> beforeShells) {
