@@ -16,15 +16,13 @@ package org.eclipse.ui.genericeditor.tests;
 import static org.junit.Assert.assertTrue;
 
 import java.util.ArrayList;
-import java.util.Collections;
+import java.util.Arrays;
+import java.util.Comparator;
 import java.util.Iterator;
 import java.util.List;
 
-import org.junit.Assert;
 import org.junit.Test;
 
-import org.eclipse.swt.widgets.Display;
-
 import org.eclipse.core.commands.Command;
 
 import org.eclipse.jface.text.ITextViewer;
@@ -33,7 +31,7 @@ import org.eclipse.jface.text.source.Annotation;
 import org.eclipse.jface.text.source.IAnnotationModel;
 import org.eclipse.jface.text.source.projection.ProjectionAnnotation;
 import org.eclipse.jface.text.source.projection.ProjectionViewer;
-import org.eclipse.jface.text.tests.util.DisplayHelper;
+import org.eclipse.ui.tests.harness.util.DisplayHelper;
 
 import org.eclipse.ui.commands.ICommandService;
 import org.eclipse.ui.genericeditor.tests.contributions.EnabledPropertyTester;
@@ -50,53 +48,45 @@ public class FoldingTest extends AbstratGenericEditorTest {
 	@Test
 	public void testDefaultIndentFoldingOneFold() throws Exception {
 		createAndOpenFile("bar.xml", "<a>\n b</a>");
-		checkFolding(pos(0, 10));
+		assertFoldingAsync(pos(0, 10));
 	}
 
 	@Test
 	public void testDefaultIndentFoldingTwoFold() throws Exception {
 		createAndOpenFile("bar.xml", "<a>\n <b>\n  c\n </b>\n</a>");
-		checkFolding(pos(0, 19), pos(4, 9));
+		assertFoldingAsync(pos(0, 19), pos(4, 9));
 	}
 
 	@Test
 	public void testCustomFoldingReconciler() throws Exception {
 		createAndOpenFile("bar.txt", "<a>\n <b>\n  c\n </b>\n</a>\n");
-		checkFolding(pos(0, 24), pos(5, 14));
+		assertFoldingAsync(pos(0, 24), pos(5, 14));
 	}
 
 	@Test
 	public void testEnabledWhenCustomFoldingReconciler() throws Exception {
 		EnabledPropertyTester.setEnabled(true);
 		createAndOpenFile("enabledWhen.txt", "<a>\n <b>\n  c\n </b>\n</a>\n");
-		checkFolding(pos(0, 24), pos(5, 14));
+		assertFoldingAsync(pos(0, 24), pos(5, 14));
 		cleanFileAndEditor();
 
 		EnabledPropertyTester.setEnabled(false);
 		createAndOpenFile("enabledWhen.txt", "<a>\n <b>\n  c\n </b>\n</a>\n");
-		checkFolding();
+		assertFoldingAsync();
 	}
 
 	private static Position pos(int offset, int length) {
 		return new Position(offset, length);
 	}
 
-	private void checkFolding(Position... expectedPositions) {
-		if (expectedPositions == null) {
-			expectedPositions= new Position[0];
-		}
-		waitForAnnotations(expectedPositions.length);
-		List<Annotation> folderAnnotations= getAnnotationsFromAnnotationModel();
-		Assert.assertEquals(expectedPositions.length, folderAnnotations.size());
-		List<Position> actualPositions= new ArrayList<>(expectedPositions.length);
-		for (int i= 0; i < expectedPositions.length; i++) {
-			Annotation folderAnnotation= folderAnnotations.get(i);
-			Position actualPosition= getProjectionAnnotationModel().getPosition(folderAnnotation);
-			actualPositions.add(actualPosition);
-		}
-		// Sort actual positions by offset
-		Collections.sort(actualPositions, (p1, p2) -> p1.offset - p2.offset);
-		Assert.assertArrayEquals(expectedPositions, actualPositions.toArray());
+	private void assertFoldingAsync(final Position... expectedPositions) {
+		DisplayHelper.waitForCondition(editor.getSite().getShell().getDisplay(), 5000, () -> {
+			Position[] actualPositions = getAnnotationsFromAnnotationModel().stream() //
+					.map(getProjectionAnnotationModel()::getPosition) //
+					.sorted(Comparator.comparingInt(Position::getOffset))
+					.toArray(Position[]::new);
+			return Arrays.deepEquals(actualPositions, expectedPositions);
+		});
 	}
 
 	private IAnnotationModel getProjectionAnnotationModel() {
@@ -105,15 +95,6 @@ public class FoldingTest extends AbstratGenericEditorTest {
 		return am;
 	}
 
-	private void waitForAnnotations(int count) {
-		new DisplayHelper() {
-			@Override
-			protected boolean condition() {
-				return getAnnotationsFromAnnotationModel().size() == count;
-			}
-		}.waitForCondition(Display.getDefault(), 2000);
-	}
-
 	private List<Annotation> getAnnotationsFromAnnotationModel() {
 		List<Annotation> annotationList= new ArrayList<>();
 		Iterator<Annotation> annotationIterator= getProjectionAnnotationModel().getAnnotationIterator();
